# Generated by roxygen2: do not edit by hand

export(.StatModel)
export(StatModel)
export(createPairwiseContrasts)
export(getCoef)
export(getContrast)
export(getDF)
export(getDfPosterior)
export(getFitMethod)
export(getModel)
export(getSigma)
export(getSigmaPosterior)
export(getVar)
export(getVarPosterior)
export(getVcovUnscaled)
export(hypothesisTest)
export(hypothesisTestHurdle)
export(makeContrast)
export(msqrob)
export(msqrobAggregate)
export(msqrobCollect)
export(msqrobGlm)
export(msqrobHurdle)
export(msqrobLm)
export(msqrobLmer)
export(msqrobQB)
export(nfLogMedian)
export(nfLogMedianOfRatios)
export(plotVolcano)
export(smallestUniqueGroups)
export(topFeatures)
export(varContrast)
exportClasses(StatModel)
exportMethods(hypothesisTest)
exportMethods(hypothesisTestHurdle)
exportMethods(msqrob)
exportMethods(msqrobAggregate)
exportMethods(msqrobHurdle)
exportMethods(msqrobQB)
exportMethods(nfLogMedian)
exportMethods(nfLogMedianOfRatios)
exportMethods(show)
import(Matrix)
import(SummarizedExperiment)
import(lme4)
import(purrr)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpmapply)
importFrom(MASS,psi.huber)
importFrom(MASS,rlm)
importFrom(MultiAssayExperiment,DataFrame)
importFrom(MultiAssayExperiment,getWithColData)
importFrom(QFeatures,addAssay)
importFrom(QFeatures,addAssayLink)
importFrom(QFeatures,aggcounts)
importFrom(assertthat,assert_that)
importFrom(codetools,makeCodeWalker)
importFrom(codetools,walkCode)
importFrom(dplyr,filter)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,ylab)
importFrom(grDevices,col2rgb)
importFrom(limma,squeezeVar)
importFrom(matrixStats,colMedians)
importFrom(methods,as)
importFrom(methods,cbind2)
importFrom(methods,is)
importFrom(methods,new)
importFrom(rlang,.data)
importFrom(stats,binomial)
importFrom(stats,glm.fit)
importFrom(stats,lm.fit)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.exclude)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pt)
importFrom(stats,resid)
importFrom(stats,terms)
importFrom(stats,update.formula)
importFrom(utils,combn)
